"""AngleSlider demo page."""

import reflex as rx

import reflex_enterprise as rxe

from .common import demo


@demo(
    route="/angle-slider",
    title="Angle Slider",
    description="An angle slider is a component that allows you to select an angle.",
)
def angle_slider_page():
    """Angle slider demo page."""
    return rx.hstack(
        rxe.angle_slider(
            on_change_end=lambda v: rx.toast(f"Selected angle: {v}"),
            marks=[
                {"value": 0, "label": "0"},
                {"value": 45, "label": "45"},
                {"value": 90, "label": "90"},
                {"value": 135, "label": "135"},
                {"value": 180, "label": "180"},
                {"value": 225, "label": "225"},
                {"value": 270, "label": "270"},
                {"value": 315, "label": "315"},
            ],
        ),
        align="center",
        spacing="4",
        margin_top="25px",
    )



"""Handlers for the Reflex Enterprise AG Grid component."""

import datetime
from types import SimpleNamespace
from typing import Any, Type, TypeVar

import reflex as rx
from sqlalchemy.orm.attributes import InstrumentedAttribute
from sqlalchemy.sql.roles import WhereHavingRole
from sqlmodel import and_, not_, or_, select
from sqlmodel.sql.expression import SelectOfScalar

M = TypeVar("M", bound=rx.Model)


def handle_text_filter(value: str, filter_def: dict[str, str]) -> bool:
    """Handle text filter."""
    type = filter_def.get("type", "contains")
    filter = filter_def.get("filter", "")
    match type:
        case "contains":
            return filter in value
        case "notContains":
            return filter not in value
        case "equals":
            return value == filter
        case "notEqual":
            return value != filter
        case "startsWith":
            return value.startswith(filter)
        case "endsWith":
            return value.endswith(filter)
        case "blank":
            return not value
        case "notBlank":
            return bool(value)
        case _:
            raise TypeError(f"type {type} does not exist")


def handle_number_filter(value: int, filter_def: dict) -> bool:
    """Handle number filter."""
    type = filter_def.get("type", "equals")
    filter = filter_def.get("filter")
    match type:
        case "equals":
            return value == filter
        case "notEqual":
            return value != filter
        case "greaterThan":
            return value > filter  # pyright: ignore [reportOperatorIssue]
        case "greaterThanOrEqual":
            return value >= filter  # pyright: ignore [reportOperatorIssue]
        case "lessThan":
            return value < filter  # pyright: ignore [reportOperatorIssue]
        case "lessThanOrEqual":
            return value <= filter  # pyright: ignore [reportOperatorIssue]
        case "inRange":
            return filter <= value <= filter_def.get("filterTo")  # pyright: ignore [reportOperatorIssue, reportOptionalOperand]
        case "blank":
            return not value
        case "notBlank":
            return bool(value)
        case _:
            raise TypeError(f"type {type} does not exist")


def handle_filter_def(value: Any, filter_def: dict) -> bool:
    """Handle filter definition."""
    if not filter_def:
        return True

    match filter_def.get("operator", "").lower():
        case "and":
            return all(
                handle_filter_def(value, sub_filter)
                for sub_filter in filter_def.get("conditions", [])
            )
        case "or":
            return any(
                handle_filter_def(value, sub_filter)
                for sub_filter in filter_def.get("conditions", [])
            )
        case _:
            pass

    match filter_def.get("filterType", "text"):
        case "text":
            return handle_text_filter(value, filter_def)
        case "number":
            return handle_number_filter(value, filter_def)
        case _:
            return False


def handle_filter_model(row: list, filter_model: dict) -> bool:
    """Handle filter model."""
    if not filter_model:
        return True
    field, filter_def = None, None
    try:
        for field, filter_def in filter_model.items():
            if not handle_filter_def(row[field], filter_def):
                return False
    except Exception as e:
        print(f"Error filtering {field} of {row}: {e}")  # noqa: T201
        return False
    return True


_sql_operations = {
    "and": and_,
    "or": or_,
}


def where_text_filter(
    value: InstrumentedAttribute, filter_def: dict[str, str]
) -> WhereHavingRole:
    """Where filter for text."""
    type = filter_def.get("type", "contains")
    filter = filter_def.get("filter", "")
    if type == "contains":
        return value.contains(filter)
    if type == "notContains":
        return not_(value.contains(filter))
    if type == "equals":
        return value == filter
    if type == "notEqual":
        return value != filter
    if type == "startsWith":
        return value.startswith(filter)
    if type == "endsWith":
        return value.endswith(filter)
    if type == "blank":
        return or_(value == None, value == "")  # noqa: E711
    if type == "notBlank":
        return and_(value != None, value != "")  # noqa: E711
    raise TypeError(f"type {type} does not exist")


def where_number_filter(
    value: InstrumentedAttribute, filter_def: dict[str, str | int | float]
) -> WhereHavingRole:
    """Where filter for numbers."""
    type = filter_def.get("type", "equals")
    to_filter = None
    if filter_def.get("filterType") == "date":
        filter = datetime.datetime.fromisoformat(filter_def.get("dateFrom"))  # pyright: ignore [reportArgumentType]
        if filter_def.get("dateTo"):
            to_filter = datetime.datetime.fromisoformat(filter_def.get("dateTo"))  # pyright: ignore [reportArgumentType]
    else:
        filter = filter_def.get("filter", 0)
        if filter_def.get("filterTo"):
            to_filter = filter_def.get("filterTo")
    if type == "equals":
        return value == filter
    if type == "notEqual":
        return value != filter
    if type == "greaterThan":
        return value > filter
    if type == "greaterThanOrEqual":
        return value >= filter
    if type == "lessThan":
        return value < filter
    if type == "lessThanOrEqual":
        return value <= filter
    if type == "inRange":
        return and_(
            value >= filter,
            value <= to_filter,
        )
    if type == "blank":
        return value == None  # noqa: E711
    if type == "notBlank":
        return value != None  # noqa: E711
    if type == "true":
        return value == True  # noqa: E712
    if type == "false":
        return value == False  # noqa: E712

    raise TypeError(f"type {type} does not exist")


def where_filter_def(
    value: InstrumentedAttribute, filter_def: dict[str, Any]
) -> WhereHavingRole | None:
    """Where filter definition."""
    if not filter_def:
        return
    operator = _sql_operations.get(
        filter_def.get("operator", "").lower(),
    )
    if operator:
        return operator(
            *(
                where_filter_def(value, sub_filter)
                for sub_filter in filter_def.get("conditions", [])
            )
        )
    filter_type = filter_def.get("filterType", "text")
    if filter_type == "text":
        return where_text_filter(value, filter_def)
    if filter_type in ("number", "date", "boolean"):
        return where_number_filter(value, filter_def)


def where_filter_outer_def(
    model: Type[M], filter_def: dict[str, Any]
) -> WhereHavingRole | None:
    """Where filter definition for supporting Advanced Filter model."""
    # Top-level advanced filter
    filter_type = filter_def.get("filterType", "text")
    if filter_type == "join":
        operator = _sql_operations.get(
            filter_def.get("type", "").lower(),
        )
        if operator:
            return operator(
                *(
                    where_filter_outer_def(model, sub_filter)
                    for sub_filter in filter_def.get("conditions", [])
                )
            )
    value = getattr(model, filter_def.get("colId"))  # pyright: ignore [reportArgumentType]
    if filter_type == "text":
        return where_text_filter(value, filter_def)
    if filter_type in ("number", "date", "boolean"):
        return where_number_filter(value, filter_def)


def apply_filter_model(
    model: Type[M], filter_model: dict[str, dict[str, Any]]
) -> SelectOfScalar[M]:
    """Apply filter model to a model."""
    query = select(model)
    if "filterType" in filter_model and "type" in filter_model:
        # Top-level advanced filter
        filter_applies = where_filter_outer_def(model, filter_model)
        if filter_applies is not None:
            return query.where(filter_applies)  # pyright: ignore [reportArgumentType]
    for field, filter_def in filter_model.items():
        filter_applies = where_filter_def(
            value=getattr(model, field),
            filter_def=filter_def,
        )
        if filter_applies is not None:
            query = query.where(filter_applies)  # pyright: ignore [reportArgumentType]
    return query


def apply_sort_model(
    model: Type[M], query: SelectOfScalar[M], sort_model: list[dict[str, str]]
) -> SelectOfScalar[M]:
    """Apply sort model to a query."""
    for sort_spec in sort_model:
        field = getattr(model, sort_spec["colId"], None)
        if field is None:
            continue
        query = query.order_by(
            field.desc() if sort_spec["sort"] == "desc" else field.asc()
        )
    return query


class HandlerNamespace(SimpleNamespace):
    """Namespace for handlers."""

    text_filter = handle_text_filter



"""Control components for leaflet maps."""

from typing import Literal

from reflex.vars.base import Var

from .base import BaseLeafletComponent


class PositionControl(BaseLeafletComponent):
    """Position Mixin for Control components."""

    # The position of the control (topleft, topright, bottomleft, bottomright)
    position: Var[Literal["topleft", "topright", "bottomleft", "bottomright"]]


class ZoomControl(PositionControl):
    """ZoomControl component for leaflet."""

    tag = "ZoomControl"


class AttributionControl(PositionControl):
    """Attribution control for leaflet maps."""

    tag = "AttributionControl"


class LayersControl(PositionControl):
    """Layers control for leaflet maps.

    This component allows users to switch between different base layers and toggle overlays.
    """

    tag = "LayersControl"

    # Whether the control should be collapsed (true) or expanded (false)
    collapsed: Var[bool]


class LayersControlBaseLayer(BaseLeafletComponent):
    """Base layer option for the LayersControl.

    This component should be used as a child of LayersControl.
    """

    tag = "LayersControl.BaseLayer"

    # Display name for the layer in the control
    name: Var[str]

    # Whether the layer should be selected by default
    checked: Var[bool]


class LayersControlOverlay(BaseLeafletComponent):
    """Overlay layer option for the LayersControl.

    This component should be used as a child of LayersControl.
    """

    tag = "LayersControl.Overlay"

    # Display name for the layer in the control
    name: Var[str]

    # Whether the overlay should be visible by default
    checked: Var[bool]


class ScaleControl(PositionControl):
    """Scale control for leaflet maps.

    Displays a scale indicator showing the distance in metric/imperial units.
    """

    tag = "ScaleControl"



"""Welcome to Reflex! This file outlines the steps to create a basic app."""

import reflex as rx

import reflex_enterprise as rxe

from .basic import basic_page
from .common import DemoState, demo
from .foreach import foreach_page
from .kanban import kanban_page

__all__ = [
    "basic_page",
    "foreach_page",
    "kanban_page",
]


@demo(
    route="/",
    title="Drag and Drop Demo",
    description="A collection of examples using React-Dnd in Reflex.",
)
def index():
    return rx.flex(
        rx.foreach(
            DemoState.pages,
            lambda page: rx.card(
                rx.vstack(
                    rx.link(page.title, href=page.route),
                    rx.text(page.description),
                ),
                width="300px",
            ),
        ),
        wrap="wrap",
        spacing="3",
    )


app = rxe.App()



"""Tooltip component for displaying information on a map."""

from __future__ import annotations

from reflex.vars.base import Var

from .base import BaseLeafletComponent
from .types import LatLng


class Tooltip(BaseLeafletComponent):
    """Tooltip component for displaying information on a map."""

    tag = "Tooltip"

    # The map pane where the tooltip element will be added.
    pane: Var[str]

    # The position where the tooltip will be shown, as a LatLng object.
    position: Var[LatLng]

    # String to be shown in the attribution control for this tooltip.
    attribution: Var[str]



"""Stub file for reflex_enterprise/components/map/vectors.py"""

# ------------------- DO NOT EDIT ----------------------
# This file was generated by `reflex/utils/pyi_generator.py`!
# ------------------------------------------------------
from typing import Any, Dict, List, Mapping, Optional, Sequence, TypedDict

from reflex.components.core.breakpoints import Breakpoints
from reflex.event import EventType, PointerEventInfo
from reflex.vars.base import Var

from .base import BaseLeafletComponent
from .types import LatLng, LatLngBounds

class PathOptions(TypedDict, total=False):
    stroke: bool
    color: str
    weight: int
    opacity: float
    lineCap: str
    lineJoin: str
    dashArray: str
    dashOffset: str
    fill: bool
    fillColor: str
    fillOpacity: float
    fillRule: str
    className: str

def path_options(
    *,
    stroke: bool | None = None,
    color: str | None = None,
    weight: int | None = None,
    opacity: float | None = None,
    line_cap: str | None = None,
    line_join: str | None = None,
    dash_array: str | None = None,
    dash_offset: str | None = None,
    fill: bool | None = None,
    fill_color: str | None = None,
    fill_opacity: float | None = None,
    fill_rule: str | None = None,
    class_name: str | None = None,
) -> PathOptions: ...

class Circle(BaseLeafletComponent):
    @classmethod
    def create(
        cls,
        *children,
        center: LatLng | Var[LatLng] | None = None,
        radius: Var[float] | float | None = None,
        path_options: PathOptions | Var[PathOptions] | None = None,
        attribution: Var[str] | str | None = None,
        pane: Var[str] | str | None = None,
        style: Sequence[Mapping[str, Any]]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Breakpoints
        | None = None,
        key: Any | None = None,
        id: Any | None = None,
        ref: Var | None = None,
        class_name: Any | None = None,
        custom_attrs: dict[str, Var | Any] | None = None,
        on_blur: Optional[EventType[()]] = None,
        on_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_context_menu: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_double_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_focus: Optional[EventType[()]] = None,
        on_mount: Optional[EventType[()]] = None,
        on_mouse_down: Optional[EventType[()]] = None,
        on_mouse_enter: Optional[EventType[()]] = None,
        on_mouse_leave: Optional[EventType[()]] = None,
        on_mouse_move: Optional[EventType[()]] = None,
        on_mouse_out: Optional[EventType[()]] = None,
        on_mouse_over: Optional[EventType[()]] = None,
        on_mouse_up: Optional[EventType[()]] = None,
        on_scroll: Optional[EventType[()]] = None,
        on_scroll_end: Optional[EventType[()]] = None,
        on_unmount: Optional[EventType[()]] = None,
        **props,
    ) -> "Circle":
        """Create the Reflex Enterprise component."""
        ...

class CircleMarker(BaseLeafletComponent):
    @classmethod
    def create(
        cls,
        *children,
        center: LatLng | Var[LatLng] | None = None,
        radius: Var[float] | float | None = None,
        path_options: PathOptions | Var[PathOptions] | None = None,
        attribution: Var[str] | str | None = None,
        pane: Var[str] | str | None = None,
        style: Sequence[Mapping[str, Any]]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Breakpoints
        | None = None,
        key: Any | None = None,
        id: Any | None = None,
        ref: Var | None = None,
        class_name: Any | None = None,
        custom_attrs: dict[str, Var | Any] | None = None,
        on_blur: Optional[EventType[()]] = None,
        on_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_context_menu: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_double_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_focus: Optional[EventType[()]] = None,
        on_mount: Optional[EventType[()]] = None,
        on_mouse_down: Optional[EventType[()]] = None,
        on_mouse_enter: Optional[EventType[()]] = None,
        on_mouse_leave: Optional[EventType[()]] = None,
        on_mouse_move: Optional[EventType[()]] = None,
        on_mouse_out: Optional[EventType[()]] = None,
        on_mouse_over: Optional[EventType[()]] = None,
        on_mouse_up: Optional[EventType[()]] = None,
        on_scroll: Optional[EventType[()]] = None,
        on_scroll_end: Optional[EventType[()]] = None,
        on_unmount: Optional[EventType[()]] = None,
        **props,
    ) -> "CircleMarker":
        """Create the Reflex Enterprise component."""
        ...

class Polyline(BaseLeafletComponent):
    @classmethod
    def create(
        cls,
        *children,
        positions: List[LatLng] | Var[List[LatLng]] | None = None,
        path_options: PathOptions | Var[PathOptions] | None = None,
        attribution: Var[str] | str | None = None,
        pane: Var[str] | str | None = None,
        style: Sequence[Mapping[str, Any]]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Breakpoints
        | None = None,
        key: Any | None = None,
        id: Any | None = None,
        ref: Var | None = None,
        class_name: Any | None = None,
        custom_attrs: dict[str, Var | Any] | None = None,
        on_blur: Optional[EventType[()]] = None,
        on_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_context_menu: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_double_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_focus: Optional[EventType[()]] = None,
        on_mount: Optional[EventType[()]] = None,
        on_mouse_down: Optional[EventType[()]] = None,
        on_mouse_enter: Optional[EventType[()]] = None,
        on_mouse_leave: Optional[EventType[()]] = None,
        on_mouse_move: Optional[EventType[()]] = None,
        on_mouse_out: Optional[EventType[()]] = None,
        on_mouse_over: Optional[EventType[()]] = None,
        on_mouse_up: Optional[EventType[()]] = None,
        on_scroll: Optional[EventType[()]] = None,
        on_scroll_end: Optional[EventType[()]] = None,
        on_unmount: Optional[EventType[()]] = None,
        **props,
    ) -> "Polyline":
        """Create the Reflex Enterprise component."""
        ...

class Polygon(BaseLeafletComponent):
    @classmethod
    def create(
        cls,
        *children,
        positions: List[LatLng] | Var[List[LatLng]] | None = None,
        path_options: PathOptions | Var[PathOptions] | None = None,
        attribution: Var[str] | str | None = None,
        pane: Var[str] | str | None = None,
        style: Sequence[Mapping[str, Any]]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Breakpoints
        | None = None,
        key: Any | None = None,
        id: Any | None = None,
        ref: Var | None = None,
        class_name: Any | None = None,
        custom_attrs: dict[str, Var | Any] | None = None,
        on_blur: Optional[EventType[()]] = None,
        on_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_context_menu: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_double_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_focus: Optional[EventType[()]] = None,
        on_mount: Optional[EventType[()]] = None,
        on_mouse_down: Optional[EventType[()]] = None,
        on_mouse_enter: Optional[EventType[()]] = None,
        on_mouse_leave: Optional[EventType[()]] = None,
        on_mouse_move: Optional[EventType[()]] = None,
        on_mouse_out: Optional[EventType[()]] = None,
        on_mouse_over: Optional[EventType[()]] = None,
        on_mouse_up: Optional[EventType[()]] = None,
        on_scroll: Optional[EventType[()]] = None,
        on_scroll_end: Optional[EventType[()]] = None,
        on_unmount: Optional[EventType[()]] = None,
        **props,
    ) -> "Polygon":
        """Create the Reflex Enterprise component."""
        ...

class Rectangle(BaseLeafletComponent):
    @classmethod
    def create(
        cls,
        *children,
        bounds: LatLngBounds | Var[LatLngBounds] | None = None,
        path_options: PathOptions | Var[PathOptions] | None = None,
        attribution: Var[str] | str | None = None,
        pane: Var[str] | str | None = None,
        style: Sequence[Mapping[str, Any]]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Breakpoints
        | None = None,
        key: Any | None = None,
        id: Any | None = None,
        ref: Var | None = None,
        class_name: Any | None = None,
        custom_attrs: dict[str, Var | Any] | None = None,
        on_blur: Optional[EventType[()]] = None,
        on_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_context_menu: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_double_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_focus: Optional[EventType[()]] = None,
        on_mount: Optional[EventType[()]] = None,
        on_mouse_down: Optional[EventType[()]] = None,
        on_mouse_enter: Optional[EventType[()]] = None,
        on_mouse_leave: Optional[EventType[()]] = None,
        on_mouse_move: Optional[EventType[()]] = None,
        on_mouse_out: Optional[EventType[()]] = None,
        on_mouse_over: Optional[EventType[()]] = None,
        on_mouse_up: Optional[EventType[()]] = None,
        on_scroll: Optional[EventType[()]] = None,
        on_scroll_end: Optional[EventType[()]] = None,
        on_unmount: Optional[EventType[()]] = None,
        **props,
    ) -> "Rectangle":
        """Create the Reflex Enterprise component."""
        ...

class SVGOverlay(BaseLeafletComponent):
    @classmethod
    def create(
        cls,
        *children,
        bounds: LatLngBounds | Var[LatLngBounds] | None = None,
        attributes: Dict[str, str] | Var[Dict[str, str]] | None = None,
        attribution: Var[str] | str | None = None,
        pane: Var[str] | str | None = None,
        style: Sequence[Mapping[str, Any]]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Breakpoints
        | None = None,
        key: Any | None = None,
        id: Any | None = None,
        ref: Var | None = None,
        class_name: Any | None = None,
        custom_attrs: dict[str, Var | Any] | None = None,
        on_blur: Optional[EventType[()]] = None,
        on_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_context_menu: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_double_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_focus: Optional[EventType[()]] = None,
        on_mount: Optional[EventType[()]] = None,
        on_mouse_down: Optional[EventType[()]] = None,
        on_mouse_enter: Optional[EventType[()]] = None,
        on_mouse_leave: Optional[EventType[()]] = None,
        on_mouse_move: Optional[EventType[()]] = None,
        on_mouse_out: Optional[EventType[()]] = None,
        on_mouse_over: Optional[EventType[()]] = None,
        on_mouse_up: Optional[EventType[()]] = None,
        on_scroll: Optional[EventType[()]] = None,
        on_scroll_end: Optional[EventType[()]] = None,
        on_unmount: Optional[EventType[()]] = None,
        **props,
    ) -> "SVGOverlay":
        """Create the Reflex Enterprise component."""
        ...



"""Tree component for displaying nested data structures."""

from __future__ import annotations

from typing import Any, TypedDict

import reflex as rx

from reflex_enterprise.components.mantine.base import MantineCoreBase


class TreeNodeData(TypedDict):
    """Data structure for a node in the Tree."""

    value: str
    label: Any
    children: list[NestedTreeNode | LeafNode]


class NestedTreeNode(TypedDict):
    """Type for a node in the nested tree structure."""

    value: str
    label: Any
    children: list[NestedTreeNode2 | LeafNode]


class NestedTreeNode2(TypedDict):
    """Type for a node in the nested tree structure."""

    value: str
    label: Any
    children: list[NestedTreeNode3 | LeafNode]


class NestedTreeNode3(TypedDict):
    """Type for a node in the nested tree structure."""

    value: str
    label: Any
    children: list[NestedTreeNode4 | LeafNode]


class NestedTreeNode4(TypedDict):
    """Type for a node in the nested tree structure."""

    value: str
    label: Any
    children: list[LeafNode]


class LeafNode(TypedDict):
    """Type for a leaf node in the tree structure."""

    value: str
    label: Any


# Tree component
class Tree(MantineCoreBase):
    """Tree component to display nested data."""

    tag = "Tree"

    # Data for the tree structure. Expects a list of TreeNodeData objects or dicts.
    data: rx.Var[list[TreeNodeData]]

    # Key of the object property used for node label. Defaults to 'label'.
    label_key: rx.Var[str]

    # Indentation multiplier in px. Defaults to 0.
    level: rx.Var[int]

    # Key of the object property used for node value. Defaults to 'value'.
    value_key: rx.Var[str]

    # Controlled state for expanded nodes (list of node values).
    value: rx.Var[list[str]]

    # Initial state for uncontrolled expanded nodes.
    default_value: rx.Var[list[str]]

    # Event handler called when a node is clicked. Passes the node data (as dict).
    on_node_click: rx.EventHandler[lambda node_dict: [node_dict]]

    # Event handler called when a node is collapsed. Passes the node data (as dict).
    on_node_collapse: rx.EventHandler[lambda node_dict: [node_dict]]

    # Event handler called when a node is expanded. Passes the node data (as dict).
    on_node_expand: rx.EventHandler[lambda node_dict: [node_dict]]

    # Event handler called when the expanded nodes change (controlled mode). Passes the new list of expanded node values.
    on_change: rx.EventHandler[lambda value_list: [value_list]]


# Create alias for the Tree component
tree = Tree.create



"""Anchor demo page."""

import reflex as rx

import reflex_enterprise as rxe

from .common import demo


@demo(
    route="/anchor",
    title="Anchor",
    description="An anchor is a component that allows you to link to a specific page.",
)
def anchor_page():
    """Anchor demo page."""
    return rx.hstack(
        rxe.anchor(
            "Anchor",
            href="https://build.reflex.dev/",
            target="_blank",
        ),
        align="center",
        spacing="4",
    )



"""Creating custom nodes is as easy as building a regular React component and passing it to nodeTypes.

Since they’re standard React components, you can display any content and implement any functionality you need. Plus, you’ll have access to a range of props that allow you to extend and customize the default node behavior.
For more details, check out our [Custom Nodes guide](https://reactflow.dev/learn/customization/custom-nodes)."""

from typing import Any

import reflex as rx

import reflex_enterprise as rxe
from reflex_enterprise.components.flow.types import Connection, Edge, Node


class CustomNodeState(rx.State):
    bg_color: rx.Field[str] = rx.field(default="#c9f1dd")
    nodes: rx.Field[list[Node]] = rx.field(
        default_factory=lambda: [
            {
                "id": "1",
                "type": "input",
                "data": {"label": "An input node"},
                "position": {"x": 0, "y": 50},
                "sourcePosition": "right",
            },
            {
                "id": "2",
                "type": "selectorNode",
                "data": {
                    "color": "#c9f1dd",
                },
                "position": {"x": 300, "y": 50},
            },
            {
                "id": "3",
                "type": "output",
                "data": {"label": "Output A"},
                "position": {"x": 650, "y": 25},
                "targetPosition": "left",
            },
            {
                "id": "4",
                "type": "output",
                "data": {"label": "Output B"},
                "position": {"x": 650, "y": 100},
                "targetPosition": "left",
            },
        ]
    )
    edges: rx.Field[list[Edge]] = rx.field(
        default_factory=lambda: [
            {
                "id": "e1-2",
                "source": "1",
                "target": "2",
                "animated": True,
            },
            {
                "id": "e2a-3",
                "source": "2",
                "target": "3",
                "animated": True,
            },
            {
                "id": "e2b-4",
                "source": "2",
                "target": "4",
                "animated": True,
            },
        ]
    )

    @rx.event
    def on_change_color(self, color: str):
        self.nodes = [
            node
            if node["id"] != "2" or "data" not in node
            else {**node, "data": {**node["data"], "color": color}}
            for node in self.nodes
        ]
        self.bg_color = color

    @rx.event
    def set_nodes(self, nodes: list[Node]):
        self.nodes = nodes

    @rx.event
    def set_edges(self, edges: list[Edge]):
        self.edges = edges


@rx.memo
def color_selector_node(data: rx.Var[dict[str, Any]], isConnectable: rx.Var[bool]):  # noqa: N803
    data = data.to(dict)
    return rx.fragment(
        rxe.flow.handle(
            type="target",
            position="left",
            on_connect=lambda params: rx.console_log(f"handle onConnect {params}"),
            is_connectable=isConnectable,
        ),
        rx.el.div(
            "Custom Color Picker Node: ",
            rx.el.strong(data["color"]),
        ),
        rx.el.input(
            class_name="nodrag",
            type="color",
            on_change=CustomNodeState.on_change_color,
            default_value=data["color"],
        ),
        rxe.flow.handle(
            type="source",
            position="right",
            is_connectable=isConnectable,
        ),
    )


def node_stroke_color(node: rx.vars.ObjectVar[Node]):
    return rx.match(
        node["type"],
        ("input", "#0041d0"),
        (
            "selectorNode",
            CustomNodeState.bg_color,
        ),
        ("output", "#ff0072"),
        None,
    )


def node_color(node: rx.vars.ObjectVar[Node]):
    return rx.match(
        node["type"],
        (
            "selectorNode",
            CustomNodeState.bg_color,
        ),
        "#fff",
    )


@rx.page(route="/nodes/custom-node", title="Custom Node Demo")
def custom_node():
    return rx.box(
        rxe.flow(
            rxe.flow.background(bg_color=CustomNodeState.bg_color),
            rxe.flow.mini_map(
                node_stroke_color=rx.vars.function.ArgsFunctionOperation.create(
                    ("node",), node_stroke_color(rx.Var("node").to(Node))
                ),
                node_color=rx.vars.function.ArgsFunctionOperation.create(
                    ("node",), node_color(rx.Var("node").to(Node))
                ),
            ),
            rxe.flow.controls(),
            nodes=CustomNodeState.nodes,
            edges=CustomNodeState.edges,
            on_nodes_change=lambda changes: CustomNodeState.set_nodes(
                rxe.flow.util.apply_node_changes(CustomNodeState.nodes, changes)
            ),
            on_edges_change=lambda changes: CustomNodeState.set_edges(
                rxe.flow.util.apply_edge_changes(CustomNodeState.edges, changes)
            ),
            on_connect=lambda connection: CustomNodeState.set_edges(
                rxe.flow.util.add_edge(
                    connection.to(dict).merge({"animated": True}).to(Connection),
                    CustomNodeState.edges,
                )
            ),
            node_types={"selectorNode": color_selector_node},
            color_mode="light",
            snap_grid=(20, 20),
            default_viewport={"x": 0, "y": 0, "zoom": 1.5},
            snap_to_grid=True,
            attribution_position="bottom-left",
            fit_view=True,
        ),
        height="100vh",
        width="100vw",
    )



"""RingProgress component for displaying progress in a circular shape."""

from typing import Any, TypedDict

import reflex as rx

from reflex_enterprise.components.mantine.base import MantineCoreBase


class Section(TypedDict, total=False):
    """Props for a section in the RingProgress component."""

    color: str
    value: float
    tooltip: str


class RingProgress(MantineCoreBase):
    """Displays progress in a circular shape."""

    tag = "RingProgress"

    # Text displayed in the center of the ring
    label: rx.Var[Any]

    # Sets whether the edges of the progress circle are rounded
    round_caps: rx.Var[bool]

    # Ring sections data
    sections: rx.Var[list[Section]]

    # Width and height of the progress ring
    size: rx.Var[float | str]

    # Ring thickness
    thickness: rx.Var[float]

    # Ring color from theme.colors
    color: rx.Var[str | list[str]]

    # Color of the root element, specified as theme color or CSS color value
    root_color: rx.Var[str]


ring_progress = RingProgress.create



"""Mantine base class."""

from typing import Literal

from reflex.assets import asset
from reflex.vars.base import Var

from reflex_enterprise.components.component import Component, ComponentEnterprise

PACKAGE_NAME = "@mantine/core"
PACKAGE_VERSION = "8.3.9"

MantineSize = Literal["xs", "sm", "md", "lg", "xl"]

provider_path = asset(path="mantine_provider.js", shared=True)
public_provider_path = "$/public/" + provider_path


class MemoizedMantineProvider(Component):
    """Next-themes integration for radix themes components."""

    library = public_provider_path
    tag = "MemoizedMantineProvider"
    is_default = True


# Base interface for Mantine components
class MantineCoreBase(ComponentEnterprise):
    """Base class for Mantine components."""

    library = f"{PACKAGE_NAME}@{PACKAGE_VERSION}"

    def add_imports(self):
        """Add import for Mantine components."""
        return {
            "": ["@mantine/core/styles.css"],
        }

    @staticmethod
    def _get_app_wrap_components() -> dict[tuple[int, str], Component]:
        return {
            (44, "MantineProvider"): MemoizedMantineProvider.create(),
        }


class MantineLeftSection(ComponentEnterprise):
    """MantineLeftSection component for Mantine."""

    # Content section rendered on the left side of the input
    left_section: Var[Component | str]

    # Props for the left section
    left_section_props: Var[dict]

    # Width of the left section
    left_section_width: Var[float | str]

    # Controls pointer events on the left section
    left_section_pointer_events: Var[str]


class MantineRightSection(ComponentEnterprise):
    """MantineRightSection component for Mantine."""

    # Content section rendered on the right side of the input
    right_section: Var[Component | str]

    # Props for the right section
    right_section_props: Var[dict]

    # Width of the right section
    right_section_width: Var[float | str]

    # Controls pointer events on the right section
    right_section_pointer_events: Var[str]


class BaseMantineInput(MantineCoreBase, MantineLeftSection, MantineRightSection):
    """BaseInput component for Mantine."""

    # Determines whether the clear button should be displayed in the right section when the component has value, false by default
    clearable: Var[bool]

    # Placeholder text displayed when the input is empty
    placeholder: Var[str]

    # Allow multiline input
    multiline: Var[bool]

    # Set the input as read-only
    read_only: Var[bool]


class MantineDescriptionProps(MantineCoreBase):
    """BaseDescription component for Mantine."""

    # Description text displayed below the input
    description: Var[Component | str]

    # Props passed down to the description component
    description_props: Var[dict]


class MantineErrorProps(MantineCoreBase):
    """BaseError component for Mantine."""

    # Error message displayed below the input
    error: Var[Component | str | bool]

    # Props passed down to the error component
    error_props: Var[dict]


class MantineLabelProps(MantineCoreBase):
    """BaseLabel component for Mantine."""

    # Label text displayed above the input
    label: Var[Component | str]

    # Props passed down to the label component
    label_props: Var[dict]



"""Test for column definition in ag-grid."""

from reflex.vars import FunctionVar, Var
from reflex.vars.function import ArgsFunctionOperation

from reflex_enterprise.components.ag_grid.resources import ColumnDef


def test_column_def():
    """Test basic column definition."""
    col = ColumnDef(field="name", header_name="Name")
    assert col.dict() == {"field": "name", "headerName": "Name"}


def test_column_def_camel_case():
    """Test column definition with camel case."""
    dict = {
        "field": "name",
        "headerName": "Name",
    }
    col = ColumnDef(**dict)  # pyright: ignore[reportArgumentType]
    assert col.dict() == {"field": "name", "headerName": "Name"}


def test_column_def_value_formatter():
    """Test column definition with value formatter."""
    format_expr = "params.value + '?'"

    col = ColumnDef(
        field="name",
        header_name="Name",
        value_formatter=format_expr,
    ).dict()
    assert isinstance(col["valueFormatter"], FunctionVar)
    assert str(col["valueFormatter"]) == f"((params) => {format_expr})"

    col2 = ColumnDef(
        field="name",
        header_name="Name",
        value_formatter={
            "function": format_expr,
        },
    ).dict()
    assert isinstance(col2["valueFormatter"], FunctionVar)
    assert str(col2["valueFormatter"]) == f"((params) => {format_expr})"

    custom_format_expr = "foobar.value + '?'"
    col3 = ColumnDef(
        field="name",
        header_name="Name",
        value_formatter=ArgsFunctionOperation.create(
            args_names=["foobar"],
            return_expr=Var(custom_format_expr),
        ),
    ).dict()
    assert isinstance(col3["valueFormatter"], FunctionVar)
    assert str(col3["valueFormatter"]) == f"((foobar) => {custom_format_expr})"



"""OIDC helpers: metadata, JWKS, and JWT verification."""

import base64
import hashlib
import logging
from collections.abc import Awaitable
from typing import Any

import httpx
from joserfc import jwk, jwt

from reflex_enterprise.auth.oidc.types import (
    AsyncHTTPClientProtocol,
    AsyncHTTPResponse,
    AsyncHTTPResponseContext,
)

# Simple cached OIDC metadata + JWKS loader
_OIDC_CACHE: dict[str, dict] = {}


# Default HTTP client for OIDC operations, can be overridden by ConfigMixin or passed in for testing
class HTTPXResponseWrapper(AsyncHTTPResponse):
    """Wrapper for httpx.Response to conform to AsyncHTTPResponse protocol."""

    def __init__(self, response: httpx.Response):
        """Initialize the wrapper with an httpx.Response."""
        self._response = response
        self.status = response.status_code

    @property
    def ok(self) -> bool:
        """Indicates if the HTTP response was successful (status code 2xx)."""
        return self._response.is_success

    def raise_for_status(self) -> Any:
        """Raise an exception if the HTTP response indicates an error."""
        return self._response.raise_for_status()

    async def json(self) -> Any:
        """Parse and return the JSON content of the HTTP response."""
        return self._response.json()


class HTTPXResponseContext(AsyncHTTPResponseContext):
    """Async context manager for httpx.Response to conform to AsyncHTTPResponseContext protocol."""

    def __init__(self, response_coro: Awaitable[httpx.Response]):
        """Initialize the context manager with a coroutine that yields an httpx.Response."""
        self._response_coro = response_coro
        self._response = None

    async def __aenter__(self) -> HTTPXResponseWrapper:
        """Enter the asynchronous context manager."""
        self._response = await self._response_coro
        return HTTPXResponseWrapper(self._response)

    async def __aexit__(
        self,
        exc_type: type[BaseException] | None,
        exc: BaseException | None,
        tb: Any,
    ) -> None:
        """Exit the asynchronous context manager."""
        if self._response is not None:
            await self._response.aclose()


class AsyncHTTPXClient(AsyncHTTPClientProtocol):
    """AsyncHTTPClientProtocol implementation using httpx."""

    def __init__(self, client: httpx.AsyncClient):
        """Initialize the AsyncHTTPXClient."""
        self._client = client

    def get(
        self,
        url: str,
        *,
        headers: dict[str, str] | None = None,
    ) -> AsyncHTTPResponseContext:
        """Perform an asynchronous HTTP GET request."""
        return HTTPXResponseContext(self._client.get(url, headers=headers))

    def post(
        self,
        url: str,
        *,
        headers: dict[str, str] | None = None,
        data: Any = None,
    ) -> AsyncHTTPResponseContext:
        """Perform an asynchronous HTTP POST request."""
        return HTTPXResponseContext(self._client.post(url, headers=headers, data=data))


DEFAULT_HTTP_CLIENT = AsyncHTTPXClient(httpx.AsyncClient())


logger = logging.getLogger(__name__)


class ValidationError(ValueError):
    """Raised when token validation fails."""


async def _fetch_oidc_metadata(
    issuer: str, client: AsyncHTTPClientProtocol = DEFAULT_HTTP_CLIENT
) -> dict:
    key = f"metadata:{issuer}"
    if key in _OIDC_CACHE:
        return _OIDC_CACHE[key]
    url = issuer.rstrip("/") + "/.well-known/openid-configuration"
    async with client.get(url) as resp:
        resp.raise_for_status()
        md = await resp.json()
    _OIDC_CACHE[key] = md
    return md


async def _fetch_jwks(
    jwks_uri: str, client: AsyncHTTPClientProtocol = DEFAULT_HTTP_CLIENT
) -> dict:
    key = f"jwks:{jwks_uri}"
    if key in _OIDC_CACHE:
        return _OIDC_CACHE[key]
    async with client.get(jwks_uri) as resp:
        resp.raise_for_status()
        jwks = await resp.json()
    _OIDC_CACHE[key] = jwks
    return jwks


async def issuer_endpoint(
    issuer_uri: str, service: str, client: AsyncHTTPClientProtocol = DEFAULT_HTTP_CLIENT
) -> str:
    """Fetch an endpoint URL (authorization/token/userinfo/etc) from OIDC metadata."""
    try:
        return (await _fetch_oidc_metadata(issuer_uri, client=client))[service]
    except KeyError:
        return ""


async def verify_jwt(
    token_json: str,
    issuer: str,
    audience: str | list[str] | None = None,
    nonce: str | None = None,
    at_hash: str | None = None,
    client: AsyncHTTPClientProtocol = DEFAULT_HTTP_CLIENT,
) -> jwt.Token:
    """Verify a JWT using OIDC discovery and JWKS per Microsoft docs.

    Returns:
        The verified JWT token.
    """
    # Fetch metadata and keys from well-known endpoint.
    md = await _fetch_oidc_metadata(issuer, client=client)
    jwks_uri = md.get("jwks_uri")
    if not jwks_uri:
        raise RuntimeError("jwks_uri not found in oidc metadata")
    jwks = await _fetch_jwks(jwks_uri, client=client)

    # Build JsonWebKey set from HTTP response/cache.
    key_set = jwk.KeySet.import_key_set(jwks)  # pyright: ignore[reportArgumentType]

    # Decode and validate the JWT signature.
    token = jwt.decode(token_json, key_set)

    stripped_issuer = issuer.rstrip("/")

    # Validate specific claims, if provided.
    iss_claim_request: jwt.ClaimsOption = {
        "essential": True,
        "values": [stripped_issuer, stripped_issuer + "/"],
    }
    aud_claim_request: jwt.ClaimsOption = {"essential": False}
    if audience is not None:
        aud_claim_request["essential"] = True
        if isinstance(audience, str):
            aud_claim_request["value"] = audience
        else:
            aud_claim_request["values"] = audience

    nonce_claim_request: jwt.ClaimsOption = {"essential": False}
    if nonce is not None:
        nonce_claim_request["essential"] = True
        nonce_claim_request["value"] = nonce

    at_hash_claim_request: jwt.ClaimsOption = {"essential": False}
    if at_hash:
        at_hash_claim_request["value"] = at_hash

    logger.debug(
        f"Verifying JWT claims: iss={iss_claim_request}, aud={aud_claim_request}, nonce={nonce_claim_request}, at_hash={at_hash_claim_request}"
    )

    claims_requests = jwt.JWTClaimsRegistry(
        iss=iss_claim_request,
        aud=aud_claim_request,
        nonce=nonce_claim_request,
        at_hash=at_hash_claim_request,
    )
    claims_requests.validate(token.claims)
    return token


def compute_at_hash(access_token: str, alg: str | None) -> str:
    """Compute at_hash value per OIDC spec for a given access token and alg."""
    if not alg:
        alg = "RS256"
    # map alg to hash
    if alg.endswith("256"):
        h = hashlib.sha256(access_token.encode("utf-8")).digest()
    elif alg.endswith("384"):
        h = hashlib.sha384(access_token.encode("utf-8")).digest()
    elif alg.endswith("512"):
        h = hashlib.sha512(access_token.encode("utf-8")).digest()
    else:
        h = hashlib.sha256(access_token.encode("utf-8")).digest()
    # left-most half
    half = h[: len(h) // 2]
    at_hash = base64.urlsafe_b64encode(half).decode("ascii").rstrip("=")
    return at_hash



import sys
import uuid
from unittest.mock import AsyncMock, Mock

import pytest
from reflex.istate.data import RouterData
from reflex.state import BaseState, _substate_key
from starlette.datastructures import Headers
from starlette.requests import Request
from starlette.routing import Route

from reflex_enterprise.app import AppEnterprise
from reflex_enterprise.auth.cookie import HTTPCookie, sync_cookies


@pytest.fixture
def mock_http_cookie_notify_sync(monkeypatch):
    monkeypatch.setattr(HTTPCookie, "notify_sync", Mock())
    return HTTPCookie.notify_sync


def test_bind_cookie_to_state_class():
    """Test binding HTTPCookie to a State subclass."""

    class TestState(BaseState):
        """A test state with a HTTPCookie descriptors."""

        _test_cookie: str = HTTPCookie("test_value", name="browser_name", max_age=100)
        _encoded_cookie: str = HTTPCookie(b"utf-8", "utf-8")
        _encoded_errors_cookie: str = HTTPCookie("😅".encode(), "ascii", "replace")

    assert isinstance(TestState._test_cookie, HTTPCookie)
    assert TestState._test_cookie == "test_value"
    assert TestState._test_cookie.state is TestState
    assert TestState._test_cookie.var_name == "_test_cookie"
    assert TestState._test_cookie.get_cookie_name() == "browser_name"
    assert TestState._test_cookie.max_age == 100
    assert TestState._encoded_cookie == "utf-8"
    assert TestState._encoded_errors_cookie == "����"

    assert "_test_cookie" not in TestState.backend_vars
    assert "_encoded_cookie" not in TestState.backend_vars
    assert "_encoded_errors_cookie" not in TestState.backend_vars
    assert "_test_cookie" in TestState.get_skip_vars()
    assert "_encoded_cookie" in TestState.get_skip_vars()
    assert "_encoded_errors_cookie" in TestState.get_skip_vars()


def test_bind_cookie_backend_only():
    """Test using HTTPCookie as a frontend var raises an error."""
    exp_error = TypeError
    if sys.version_info < (3, 12):
        # Older interpreters don't re-raise the inner error.
        exp_error = RuntimeError

    with pytest.raises(exp_error):

        class InvalidState(BaseState):
            """An invalid state with HTTPCookie as a frontend var."""

            invalid_cookie: str = HTTPCookie()


def test_set_get_cookie_value(mock_http_cookie_notify_sync):
    """Test setting and getting cookie values."""

    class TestState(BaseState):
        """A test state with an HTTPCookie."""

        _test_cookie: str = HTTPCookie(
            "initial_value", name="browser_name", max_age=100
        )

    state_instance = TestState()

    assert state_instance._test_cookie == "initial_value"
    assert "_test_cookie" not in state_instance.dirty_vars
    assert not TestState._test_cookie.get_is_dirty(state_instance)  # pyright: ignore[reportAttributeAccessIssue]

    state_instance._test_cookie = "new_value"
    assert state_instance._test_cookie == "new_value"
    assert "_test_cookie" in state_instance.dirty_vars
    assert TestState._test_cookie.get_is_dirty(state_instance)  # pyright: ignore[reportAttributeAccessIssue]

    mock_http_cookie_notify_sync.assert_called_once_with(state_instance)


def test_set_cookie_no_sync(mock_http_cookie_notify_sync):
    """Test setting cookie value with _sync_on_set=False does not notify sync."""

    class TestState(BaseState):
        """A test state with an HTTPCookie."""

        _test_cookie: str = HTTPCookie("initial_value", _sync_on_set=False)
        _test_cookie_explicit_no_sync: str = HTTPCookie("initial_value")

    state_instance = TestState()

    state_instance._test_cookie = "new_value_no_sync"
    assert state_instance._test_cookie == "new_value_no_sync"
    assert "_test_cookie" in state_instance.dirty_vars
    assert TestState._test_cookie.get_is_dirty(state_instance)  # pyright: ignore[reportAttributeAccessIssue]
    mock_http_cookie_notify_sync.assert_not_called()

    state_instance.dirty_vars.clear()
    state_instance.__delattr__(TestState._test_cookie.dirty_attribute_name)  # pyright: ignore[reportAttributeAccessIssue]

    # Cannot override the sync value set in the definition
    state_instance._test_cookie = HTTPCookie("new_value_try_sync", _sync_on_set=True)
    assert state_instance._test_cookie == "new_value_try_sync"
    assert "_test_cookie" in state_instance.dirty_vars
    assert TestState._test_cookie.get_is_dirty(state_instance)  # pyright: ignore[reportAttributeAccessIssue]
    mock_http_cookie_notify_sync.assert_not_called()

    state_instance.dirty_vars.clear()
    state_instance.__delattr__(TestState._test_cookie.dirty_attribute_name)  # pyright: ignore[reportAttributeAccessIssue]

    # Can disable the sync for a one-off set
    state_instance._test_cookie_explicit_no_sync = HTTPCookie(
        "new_value_no_sync_explicit",
        _sync_on_set=False,
    )
    assert state_instance._test_cookie_explicit_no_sync == "new_value_no_sync_explicit"
    assert "_test_cookie" not in state_instance.dirty_vars
    assert "_test_cookie_explicit_no_sync" in state_instance.dirty_vars
    assert TestState._test_cookie_explicit_no_sync.get_is_dirty(state_instance)  # pyright: ignore[reportAttributeAccessIssue]
    mock_http_cookie_notify_sync.assert_not_called()

    state_instance.dirty_vars.clear()
    state_instance.__delattr__(
        TestState._test_cookie_explicit_no_sync.dirty_attribute_name  # pyright: ignore[reportAttributeAccessIssue]
    )

    # The default sync behavior is working when setting just a str
    state_instance._test_cookie_explicit_no_sync = "new_value_sync"
    assert state_instance._test_cookie_explicit_no_sync == "new_value_sync"
    assert "_test_cookie" not in state_instance.dirty_vars
    assert "_test_cookie_explicit_no_sync" in state_instance.dirty_vars
    assert TestState._test_cookie_explicit_no_sync.get_is_dirty(state_instance)  # pyright: ignore[reportAttributeAccessIssue]
    mock_http_cookie_notify_sync.assert_called_once_with(state_instance)

    state_instance.dirty_vars.clear()
    state_instance.__delattr__(
        TestState._test_cookie_explicit_no_sync.dirty_attribute_name  # pyright: ignore[reportAttributeAccessIssue]
    )


def test_set_cookie_from_browser(mock_http_cookie_notify_sync):
    """Test setting cookie value from browser sync."""

    class TestState(BaseState):
        """A test state with an HTTPCookie."""

        _test_cookie: str = HTTPCookie("initial_value")

    state_instance = TestState()

    # Simulate pulling cookie value from browser
    TestState._test_cookie.set(  # pyright: ignore[reportAttributeAccessIssue]
        state_instance,
        "updated_value_from_browser",
        from_browser=True,
    )
    assert state_instance._test_cookie == "updated_value_from_browser"
    assert "_test_cookie" in state_instance.dirty_vars
    assert not TestState._test_cookie.get_is_dirty(state_instance)  # pyright: ignore[reportAttributeAccessIssue]
    mock_http_cookie_notify_sync.assert_not_called()


def test_get_cookie_from_router_data(mock_http_cookie_notify_sync):
    """Test getting cookie value from router data."""

    class TestState(BaseState):
        """A test state with an HTTPCookie."""

        _test_cookie: str = HTTPCookie("initial_value", name="browser_name")

    state_instance = TestState()
    assert state_instance._test_cookie == "initial_value"

    state_instance.router_data = {
        "headers": {
            "cookie": "other_cookie=other_value; browser_name=value_from_cookie; another_cookie=another_value"
        }
    }
    state_instance.router = RouterData.from_router_data(state_instance.router_data)
    assert state_instance._test_cookie == "value_from_cookie"
    mock_http_cookie_notify_sync.assert_not_called()

    state_instance._test_cookie = "set_explicitly"
    assert state_instance._test_cookie == "set_explicitly"
    mock_http_cookie_notify_sync.assert_called_once_with(state_instance)
    mock_http_cookie_notify_sync.reset_mock()

    # After deleting the cookie, the value from the router_data is ignored.
    del state_instance._test_cookie
    assert state_instance._test_cookie == "initial_value"
    mock_http_cookie_notify_sync.assert_called_once_with(state_instance)


@pytest.mark.usefixtures("mock_http_cookie_notify_sync")
@pytest.mark.asyncio
async def test_get_set_cookie_substates():
    """Test getting and setting cookies in substates."""

    class ParentState(BaseState):
        """A parent state with an HTTPCookie."""

        _parent_cookie: str = HTTPCookie("parent_value", name="parent_name")

    class ChildState(ParentState):
        """A child state with an HTTPCookie."""

        _child_cookie: str = HTTPCookie("child_value", name="child_name")

    class GrandChildState(ChildState):
        """A grandchild state without its own HTTPCookie."""

        pass

    assert "_parent_cookie" not in ParentState.backend_vars
    assert "_parent_cookie" not in ChildState.backend_vars
    assert "_parent_cookie" not in GrandChildState.backend_vars
    assert "_child_cookie" not in ChildState.backend_vars
    assert "_child_cookie" not in GrandChildState.backend_vars

    app = AppEnterprise(_state=ParentState)

    async with app.state_manager.modify_state(
        _substate_key(str(uuid.uuid4()), GrandChildState)
    ) as state_instance:
        # Test inherited parent cookie
        assert state_instance._parent_cookie == "parent_value"  # pyright: ignore[reportAttributeAccessIssue]
        state_instance._parent_cookie = "new_parent_value"
        assert state_instance._parent_cookie == "new_parent_value"  # pyright: ignore[reportAttributeAccessIssue]

        grandchild = await state_instance.get_state(GrandChildState)
        assert grandchild._parent_cookie == "new_parent_value"

        # Set parent cookie from substate
        grandchild._parent_cookie = "new_parent_value_in_grandchild_instance"
        assert grandchild._parent_cookie == "new_parent_value_in_grandchild_instance"
        assert (
            state_instance._parent_cookie == "new_parent_value_in_grandchild_instance"  # pyright: ignore[reportAttributeAccessIssue]
        )
        grandchild._child_cookie = "new_child_value_in_grandchild_instance"
        child = await state_instance.get_state(ChildState)
        assert child._child_cookie == "new_child_value_in_grandchild_instance"


class HTTPCookieTestState(BaseState):
    """A test state with multiple HTTPCookies."""

    _session_id: str = HTTPCookie("$", name="session_cookie", max_age=3600)
    _no_notify_cookie: str = HTTPCookie(
        "no_notify_value", max_age=3600, _sync_on_set=False
    )
    _temp_cookie: str = HTTPCookie("temp_value", name="temp_cookie", max_age=30)
    _default_name_cookie: str = HTTPCookie()


class HTTPCookieSubTestState(HTTPCookieTestState):
    """A subclass of HTTPCookieTestState to test inheritance."""

    _sub_cookie: str = HTTPCookie("sub_value", name="sub_cookie", max_age=600)


@pytest.fixture
def mock_app(monkeypatch):
    mock_app_module = Mock()
    mock_app = AppEnterprise(_state=HTTPCookieTestState)
    mock_app._event_namespace = AsyncMock()
    mock_app_module.app = mock_app
    monkeypatch.setattr(
        "reflex_enterprise.auth.cookie.get_app", lambda: mock_app_module
    )
    return mock_app


@pytest.fixture
def token():
    return str(uuid.uuid4())


@pytest.mark.asyncio
async def test_sync_cookies_from_browser(mock_app, token):
    """Test syncing cookies with the browser."""

    request = Mock(spec=Request)
    request.headers = Headers({"X-Reflex-Client-Token": token})
    request.cookies = {
        "session_cookie": "session_value_from_browser",
        "temp_cookie": "temp_value_from_browser",
        "sub_cookie": "sub_value_from_browser",
    }

    response = await sync_cookies(request)
    assert response.headers.get("Set-Cookie") is None

    state_instance = await mock_app.state_manager.get_state(
        _substate_key(token, HTTPCookieSubTestState)
    )
    assert state_instance._session_id == "session_value_from_browser"
    assert state_instance._temp_cookie == "temp_value_from_browser"
    assert state_instance._no_notify_cookie == "no_notify_value"
    assert not HTTPCookieTestState._session_id.get_is_dirty(state_instance)  # pyright: ignore[reportAttributeAccessIssue]
    assert not HTTPCookieTestState._temp_cookie.get_is_dirty(state_instance)  # pyright: ignore[reportAttributeAccessIssue]
    assert not HTTPCookieTestState._no_notify_cookie.get_is_dirty(state_instance)  # pyright: ignore[reportAttributeAccessIssue]
    sub_instance = await state_instance.get_state(HTTPCookieSubTestState)
    assert sub_instance._sub_cookie == "sub_value_from_browser"
    assert not HTTPCookieSubTestState._sub_cookie.get_is_dirty(sub_instance)  # pyright: ignore[reportAttributeAccessIssue]


@pytest.mark.asyncio
async def test_sync_cookies_to_browser(mock_app, token):
    """Test syncing cookies to the browser."""

    request = Mock(spec=Request)
    request.headers = Headers({"X-Reflex-Client-Token": token})
    request.cookies = {}

    async with mock_app.state_manager.modify_state(
        _substate_key(token, HTTPCookieTestState)
    ) as state_instance:
        state_instance._session_id = "new_session_value"
        state_instance._no_notify_cookie = "new_no_notify_value"
        state_instance._temp_cookie = HTTPCookie("new_temp_value", max_age=666)

    response = await sync_cookies(request)

    set_cookies = dict.fromkeys(response.headers.getlist("Set-Cookie"), True)
    assert set_cookies.pop(
        "session_cookie=new_session_value; HttpOnly; Max-Age=3600; Path=/; SameSite=strict; Secure"
    )
    assert set_cookies.pop(
        "auth___test_cookie___http_cookie_test_state._no_notify_cookie_rx_state_=new_no_notify_value; HttpOnly; Max-Age=3600; Path=/; SameSite=strict; Secure"
    )
    assert set_cookies.pop(
        "temp_cookie=new_temp_value; HttpOnly; Max-Age=666; Path=/; SameSite=strict; Secure"
    )
    assert not set_cookies

    state_instance = await mock_app.state_manager.get_state(
        _substate_key(token, HTTPCookieTestState)
    )
    assert not HTTPCookieTestState._session_id.get_is_dirty(state_instance)  # pyright: ignore[reportAttributeAccessIssue]
    assert not HTTPCookieTestState._temp_cookie.get_is_dirty(state_instance)  # pyright: ignore[reportAttributeAccessIssue]
    assert not HTTPCookieTestState._no_notify_cookie.get_is_dirty(state_instance)  # pyright: ignore[reportAttributeAccessIssue]


async def test_sync_cookies_bidirectional(mock_app, token):
    """Test that dirty backend cookies override browser sent cookies."""

    request = Mock(spec=Request)
    request.headers = Headers({"X-Reflex-Client-Token": token})
    request.cookies = {
        "session_cookie": "session_value_from_browser",
        "temp_cookie": "temp_value_from_browser",
    }

    async with mock_app.state_manager.modify_state(
        _substate_key(token, HTTPCookieTestState)
    ) as state_instance:
        sub_instance = await state_instance.get_state(HTTPCookieSubTestState)
        sub_instance._session_id = "new_session_value_backend"
        sub_instance._default_name_cookie = "default_name_value_backend"
        assert HTTPCookieTestState._session_id.get_is_dirty(state_instance)  # pyright: ignore[reportAttributeAccessIssue]
        assert HTTPCookieTestState._default_name_cookie.get_is_dirty(state_instance)  # pyright: ignore[reportAttributeAccessIssue]

    response = await sync_cookies(request)

    assert not HTTPCookieTestState._session_id.get_is_dirty(state_instance)  # pyright: ignore[reportAttributeAccessIssue]

    set_cookies = dict.fromkeys(response.headers.getlist("Set-Cookie"), True)
    assert set_cookies.pop(
        "session_cookie=new_session_value_backend; HttpOnly; Max-Age=3600; Path=/; SameSite=strict; Secure"
    )
    assert set_cookies.pop(
        "auth___test_cookie___http_cookie_test_state._default_name_cookie_rx_state_=default_name_value_backend; HttpOnly; Path=/; SameSite=strict; Secure"
    )
    assert not set_cookies

    state_instance = await mock_app.state_manager.get_state(
        _substate_key(token, HTTPCookieTestState)
    )
    assert state_instance._session_id == "new_session_value_backend"
    assert state_instance._temp_cookie == "temp_value_from_browser"
    assert state_instance._no_notify_cookie == "no_notify_value"
    assert state_instance._default_name_cookie == "default_name_value_backend"
    sub_instance = await state_instance.get_state(HTTPCookieSubTestState)
    assert sub_instance._sub_cookie == "sub_value"
    assert sub_instance._session_id == "new_session_value_backend"
    assert sub_instance._temp_cookie == "temp_value_from_browser"
    assert sub_instance._no_notify_cookie == "no_notify_value"
    assert sub_instance._default_name_cookie == "default_name_value_backend"


@pytest.mark.asyncio
async def test_sync_cookies_browser_reset(mock_app, token):
    """Test that cookies deleted in the browser are reset in the backend."""

    request = Mock(spec=Request)
    request.headers = Headers({"X-Reflex-Client-Token": token})
    request.cookies = {}

    async with mock_app.state_manager.modify_state(
        _substate_key(token, HTTPCookieTestState)
    ) as state_instance:
        assert state_instance._session_id == "$"
        state_instance._session_id = "new_session_value"

    response = await sync_cookies(request)

    set_cookies = dict.fromkeys(response.headers.getlist("Set-Cookie"), True)
    assert set_cookies.pop(
        "session_cookie=new_session_value; HttpOnly; Max-Age=3600; Path=/; SameSite=strict; Secure"
    )
    assert not set_cookies

    # Sync again, without a dirty cookie, expect it to be deleted.
    response = await sync_cookies(request)
    assert response.headers.get("Set-Cookie") is None
    state_instance = await mock_app.state_manager.get_state(
        _substate_key(token, HTTPCookieTestState)
    )
    assert state_instance._session_id == "$"
    assert state_instance._session_id.max_age == 0  # pyright: ignore[reportAttributeAccessIssue]


@pytest.mark.asyncio
async def test_sync_cookies_no_token():
    """Test that sync_cookies returns a response with no Set-Cookie headers if no token is provided."""

    request = Mock(spec=Request)
    request.headers = {}
    request.cookies = {
        "session_cookie": "session_value_from_browser",
        "temp_cookie": "temp_value_from_browser",
    }

    response = await sync_cookies(request)
    assert response.status_code == 400
    assert response.body == b"No client token in request"


def test_unbound_cookie_access():
    """Test that accessing attributes on a free-standing cookie fail."""

    c = HTTPCookie("value")
    with pytest.raises(RuntimeError):
        _ = c.state
    with pytest.raises(RuntimeError):
        _ = c.var_name
    with pytest.raises(RuntimeError):
        _ = c.get_cookie_name()
    with pytest.raises(RuntimeError):
        _ = c.attribute_name
    with pytest.raises(RuntimeError):
        _ = c.dirty_attribute_name


def test_out_of_tree_access():
    """Test that accessing cookie on non-owning state fails."""

    class SomeState(BaseState):
        """A test state with an HTTPCookie."""

        _session_id: str = HTTPCookie("value")

    state_instance = SomeState()

    with pytest.raises(AttributeError):
        HTTPCookieTestState._session_id.__get__(state_instance, HTTPCookieTestState)  # pyright: ignore[reportAttributeAccessIssue]


def test_ensure_handlers_registered(mock_app):
    """Test that ensure_handlers_registered works without error."""
    # Reset the registration flag for testing
    HTTPCookie._has_registered_handlers = False

    exp_route = Route(
        path="/_reflex/cookies/sync",
        endpoint=sync_cookies,
        name="sync_cookies",
        methods=["POST"],
    )
    assert exp_route not in mock_app._api.routes
    routes_before = mock_app._api.routes.copy()

    HTTPCookie.ensure_handlers_registered()
    assert exp_route in mock_app._api.routes
    routes_after_1 = mock_app._api.routes.copy()

    # Shouldn't have registered it multiple times.
    HTTPCookie.ensure_handlers_registered()
    HTTPCookie.ensure_handlers_registered()
    HTTPCookie.ensure_handlers_registered()
    assert routes_after_1 == mock_app._api.routes
    assert len(routes_before) + 1 == len(mock_app._api.routes)


@pytest.mark.asyncio
async def test_notify_send(mock_app, token):
    """Test that notify_sync sends an update to the client."""
    async with mock_app.modify_state(
        _substate_key(token, HTTPCookieTestState)
    ) as state_instance:
        state_instance.router_data = {"token": token}
        state_instance.router = RouterData.from_router_data(state_instance.router_data)
    async with mock_app.modify_state(
        _substate_key(token, HTTPCookieTestState)
    ) as state_instance:
        state_instance._session_id = "new_value"

    assert mock_app.event_namespace.emit_update.call_args.kwargs["token"] == token
    update = mock_app.event_namespace.emit_update.call_args.kwargs["update"]
    assert not update.delta
    assert update.final is None
    assert len(update.events) == 1
    event = update.events[0]
    assert event.token == token
    assert event.name == "_call_script"
    js_code = event.payload["javascript_code"]
    assert "fetch(" in js_code
    assert '"/_reflex/cookies/sync"' in js_code
    assert '"include"' in js_code
    assert '"POST"' in js_code
    assert not event.payload.get("callback")



"""Marker component for displaying a marker on a map."""

from reflex.vars.base import Var

from .base import BaseLeafletComponent
from .types import LatLng


class Marker(BaseLeafletComponent):
    """Marker component for displaying a marker on a map."""

    tag = "Marker"

    # The position of the marker
    position: Var[LatLng]

    # Whether the marker is draggable or not
    draggable: Var[bool]

    # The z-index offset of the marker
    z_index_offset: Var[int]

    # The opacity of the marker (0-1)
    opacity: Var[float]

    # The attribution text for the marker
    attribution: Var[str]



"""Helpers for listening to postMessage events from the browser window.

This module exposes a lightweight Reflex component that embeds a small
frontend script to forward window.postMessage events into Reflex event
handlers. It defines a `WindowMessage` type and the `message_listener`
factory for creating the component.
"""

from typing import Any, TypedDict

import reflex as rx
from reflex.event import passthrough_event_spec

from reflex_enterprise.components.component import ComponentEnterprise

from .funcs import CLOSE_POPUP, POST_MESSAGE_AND_CLOSE_POPUP, WINDOW_OPEN


class WindowMessage(TypedDict):
    """Type describing a message forwarded from the browser.

    Attributes:
        origin: The origin string of the message sender.
        data: The payload sent in the message.
        timestamp: A numeric timestamp included by the sender (if any).
    """

    origin: str
    data: Any
    timestamp: float


class MessageListener(ComponentEnterprise):
    """Reflex component embedding a small JS listener for window.postMessage.

    Use the `message_listener` factory to instantiate the component. The
    component accepts `allowed_origin` to restrict messages and an `on_message`
    event handler that receives a `WindowMessage` payload.
    """

    library = "$/public" + rx.asset("messageListener.js", shared=True)

    tag = "MessageListener"

    allowed_origin: str | None = None
    on_message: rx.EventHandler[passthrough_event_spec(WindowMessage)]
    # If supplied, only call on_message if the value of corresponding keys in the event data matches all regexes provided.
    filter_attributes: rx.Var[dict[str, str]]


message_listener = MessageListener.create

__all__ = [
    "CLOSE_POPUP",
    "POST_MESSAGE_AND_CLOSE_POPUP",
    "WINDOW_OPEN",
    "MessageListener",
    "WindowMessage",
    "message_listener",
]



"""Fill Handle demo for AG Grid in Reflex."""

from typing import Any, TypedDict

import reflex as rx

import reflex_enterprise as rxe

from .common import demo


class DataRow(TypedDict):
    """Data row type for AG Grid."""

    athlete: str
    age: int | None
    country: str
    year: int | None
    date: str
    sport: str
    gold: int
    silver: int
    bronze: int
    total: int


class FillHandleState(rx.State):
    """State for the fill handle demo."""

    # Sample Olympic data similar to AG Grid's example
    row_data: list[DataRow] = [
        {
            "athlete": "Michael Phelps",
            "age": 23,
            "country": "United States",
            "year": 2008,
            "date": "24/08/2008",
            "sport": "Swimming",
            "gold": 8,
            "silver": 0,
            "bronze": 0,
            "total": 8,
        },
        {
            "athlete": "Michael Phelps",
            "age": 19,
            "country": "United States",
            "year": 2004,
            "date": "29/08/2004",
            "sport": "Swimming",
            "gold": 6,
            "silver": 0,
            "bronze": 2,
            "total": 8,
        },
        {
            "athlete": "Michael Phelps",
            "age": 27,
            "country": "United States",
            "year": 2012,
            "date": "12/08/2012",
            "sport": "Swimming",
            "gold": 4,
            "silver": 2,
            "bronze": 0,
            "total": 6,
        },
        {
            "athlete": "Natalie Coughlin",
            "age": 25,
            "country": "United States",
            "year": 2008,
            "date": "24/08/2008",
            "sport": "Swimming",
            "gold": 1,
            "silver": 2,
            "bronze": 3,
            "total": 6,
        },
        {
            "athlete": "Aleksey Nemov",
            "age": 24,
            "country": "Russia",
            "year": 2000,
            "date": "01/10/2000",
            "sport": "Gymnastics",
            "gold": 2,
            "silver": 1,
            "bronze": 3,
            "total": 6,
        },
        {
            "athlete": "Alicia Coutts",
            "age": 24,
            "country": "Australia",
            "year": 2012,
            "date": "12/08/2012",
            "sport": "Swimming",
            "gold": 1,
            "silver": 3,
            "bronze": 1,
            "total": 5,
        },
        {
            "athlete": "Missy Franklin",
            "age": 17,
            "country": "United States",
            "year": 2012,
            "date": "12/08/2012",
            "sport": "Swimming",
            "gold": 4,
            "silver": 0,
            "bronze": 1,
            "total": 5,
        },
        {
            "athlete": "Ryan Lochte",
            "age": 27,
            "country": "United States",
            "year": 2012,
            "date": "12/08/2012",
            "sport": "Swimming",
            "gold": 2,
            "silver": 2,
            "bronze": 1,
            "total": 5,
        },
        {
            "athlete": "Allison Schmitt",
            "age": 22,
            "country": "United States",
            "year": 2012,
            "date": "12/08/2012",
            "sport": "Swimming",
            "gold": 3,
            "silver": 1,
            "bronze": 1,
            "total": 5,
        },
        {
            "athlete": "Natalie Coughlin",
            "age": 21,
            "country": "United States",
            "year": 2004,
            "date": "29/08/2004",
            "sport": "Swimming",
            "gold": 2,
            "silver": 2,
            "bronze": 1,
            "total": 5,
        },
    ]

    @rx.event
    def on_cell_value_changed(self, params: dict[str, Any]):
        """Handle cell value changes from fill handle operations."""
        row_index = int(params["node_id"])
        field = params["field"]
        new_value = params["newValue"]

        # Convert numeric fields appropriately
        if field in ["age", "year", "gold", "silver", "bronze", "total"]:
            try:
                new_value = int(new_value) if new_value else None
            except (ValueError, TypeError):
                new_value = None

        # Update the row data in the state
        self.row_data[row_index][field] = new_value

        return rx.toast(f"Updated {field}: {new_value}")


# Column definitions based on the AG Grid reference
column_defs = [
    {
        "field": "athlete",
        "width": 150,
        "suppress_fill_handle": True,  # Text fields typically don't use fill handle
    },
    {
        "field": "age",
        "width": 90,
        "editable": True,
    },
    {
        "field": "country",
        "width": 120,
        "suppress_fill_handle": True,  # Country names shouldn't be filled
    },
    {
        "field": "year",
        "width": 90,
        "editable": True,
    },
    {
        "field": "date",
        "width": 110,
        "editable": True,  # Enable editing and fill handle for dates
    },
    {
        "field": "sport",
        "width": 110,
        "suppress_fill_handle": True,  # Sport names shouldn't be filled
    },
    {
        "field": "gold",
        "width": 100,
        "editable": True,
        "type": "numericColumn",
    },
    {
        "field": "silver",
        "width": 100,
        "editable": True,
        "type": "numericColumn",
    },
    {
        "field": "bronze",
        "width": 100,
        "editable": True,
        "type": "numericColumn",
    },
    {
        "field": "total",
        "width": 100,
        "editable": True,
        "type": "numericColumn",
    },
]


@demo(
    route="/fill-handle",
    title="Fill Handle",
    description="Demonstrates the Fill Handle feature. Select cells and drag the fill handle (small square in bottom-right corner) to copy values or create series.",
)
def fill_handle_page():
    """Fill handle demo page."""
    return rx.vstack(
        rx.heading("Fill Handle Demo", size="6", margin_bottom="1em"),
        rx.text(
            "How to use the Fill Handle:",
            font_weight="bold",
            margin_bottom="0.5em",
        ),
        rx.unordered_list(
            rx.list_item("Select a cell or range of cells"),
            rx.list_item(
                "Look for the small square (fill handle) in the bottom-right corner"
            ),
            rx.list_item("Drag the fill handle to extend the selection"),
            rx.list_item("For numbers: Creates incremental series (1, 2, 3...)"),
            rx.list_item("For dates: Copies the date value to adjacent cells"),
            rx.list_item("For single values: Copies the value to all selected cells"),
            rx.list_item("Try with different data types (age, year, dates, medals)"),
            margin_bottom="1em",
        ),
        rx.text(
            "Note: Fill handle is disabled only for text fields like athlete, country, and sport to show selective control.",
            color="gray",
            font_size="sm",
            margin_bottom="1em",
        ),
        rxe.ag_grid(
            id="fill-handle-grid",
            column_defs=column_defs,
            row_data=FillHandleState.row_data,
            on_cell_value_changed=FillHandleState.on_cell_value_changed,
            # Configure cell selection with fill handle - single cell only
            cell_selection={
                "mode": "singleCell",  # Only allow single cell selection
                "handle": {
                    "mode": "fill",  # Enable fill handle
                    "direction": "xy",  # Allow both horizontal and vertical filling
                },
            },
            width="100%",
            height="500px",
        ),
        width="100%",
    )



"""Enterprise utilities for Reflex CLI."""

from dataclasses import dataclass
from typing import ClassVar

from reflex.config import Config


@dataclass(kw_only=True, init=False)
class ConfigEnterprise(Config):
    """Enterprise configuration class."""

    use_single_port: bool | None = None

    _prefixes: ClassVar[list[str]] = ["REFLEX_", "REFLEX_ENTERPRISE_"]

    def __init__(self, **kwargs):
        """Initialize the configuration."""
        # This is a hack due to how upstream Config/BaseConfig interact
        if "use_single_port" in kwargs:
            self.use_single_port = kwargs.pop("use_single_port")
        super().__init__(**kwargs)


Config = ConfigEnterprise



"""Navigation progress bar for Mantine."""

from reflex.vars.base import Var

from reflex_enterprise.components.component import ComponentEnterprise


class NProgress(ComponentEnterprise):
    """NProgress component for Mantine."""

    library = "@mantine/nprogress@8.3.9"

    tag = "NProgress"

    alias = "MantineNProgress"

    # Progress bar color
    color: Var[str]

    # Initial progress value, 0 by default
    initial_progress: Var[int]

    # Props to pass down to the Portal when withinPortal is true
    portal_props: Var[dict]

    # Controls height of the progress bar
    size: Var[int]

    # Step interval in ms, 500 by default
    step_interval: Var[int]

    # Component store, controls state
    store: Var[dict]

    # Determines whether the progress bar should be rendered within Portal, true by default
    within_portal: Var[bool]

    # Progressbar z-index, 9999 by default
    z_index: Var[int]



"""Stub file for reflex_enterprise/__init__.py"""
# ------------------- DO NOT EDIT ----------------------
# This file was generated by `reflex/utils/pyi_generator.py`!
# ------------------------------------------------------

from . import components
from .app import AppEnterprise as App
from .components import mantine
from .components.ag_charts import ag_chart
from .components.ag_grid import ag_grid
from .components.ag_grid.wrapper import (
    ModelWrapper,
    ModelWrapperSSRM,
    model_wrapper,
    model_wrapper_ssrm,
)
from .components.dnd import dnd
from .components.flow import flow
from .components.mantine.autocomplete import autocomplete
from .components.mantine.collapse import collapse
from .components.mantine.combobox import combobox
from .components.mantine.dates import dates
from .components.mantine.json_input import json_input
from .components.mantine.loading_overlay import loading_overlay
from .components.mantine.multi_select import multi_select
from .components.mantine.number_formatter import number_formatter
from .components.mantine.pill import pill
from .components.mantine.pills_input import pills_input
from .components.mantine.ring_progress import ring_progress
from .components.mantine.semi_circle_progress import semi_circle_progress
from .components.mantine.spoiler import spoiler
from .components.mantine.tags_input import tags_input
from .components.mantine.timeline import timeline
from .components.mantine.tree import tree
from .components.map import map
from .config import ConfigEnterprise as Config
from .utils import arrow_func
from .vars import (
    ArgsFunctionOperationPromise,
    ElementAPI,
    JSAPIVar,
    LambdaVar,
    LiteralLambdaVar,
    PassthroughAPI,
    PromiseVar,
    is_static,
    static,
)

_MANTINE_MAPPING = {
    "components.mantine.autocomplete": ["autocomplete"],
    "components.mantine.collapse": ["collapse"],
    "components.mantine.combobox": ["combobox"],
    "components.mantine.json_input": ["json_input"],
    "components.mantine.loading_overlay": ["loading_overlay"],
    "components.mantine.multi_select": ["multi_select"],
    "components.mantine.number_formatter": ["number_formatter"],
    "components.mantine.pill": ["pill"],
    "components.mantine.pills_input": ["pills_input"],
    "components.mantine.ring_progress": ["ring_progress"],
    "components.mantine.semi_circle_progress": ["semi_circle_progress"],
    "components.mantine.spoiler": ["spoiler"],
    "components.mantine.tags_input": ["tags_input"],
    "components.mantine.tree": ["tree"],
    "components.mantine.timeline": ["timeline"],
    "components.mantine.dates": ["dates"],
}
_SUBMODULES = {"components"}
_SUBMOD_ATTRS = {
    "app": [("AppEnterprise", "App")],
    "utils": ["arrow_func"],
    "config": [("ConfigEnterprise", "Config")],
    "components": ["mantine"],
    "components.ag_grid": ["ag_grid"],
    "components.ag_charts": ["ag_chart"],
    **_MANTINE_MAPPING,
    "components.ag_grid.wrapper": [
        "model_wrapper",
        "model_wrapper_ssrm",
        "ModelWrapper",
        "ModelWrapperSSRM",
    ],
    "components.dnd": ["dnd"],
    "components.flow": ["flow"],
    "components.map": ["map"],
    "vars": [
        "ArgsFunctionOperationPromise",
        "PassthroughAPI",
        "ElementAPI",
        "JSAPIVar",
        "LambdaVar",
        "LiteralLambdaVar",
        "PromiseVar",
        "is_static",
        "static",
    ],
}

__all__ = [
    "App",
    "ArgsFunctionOperationPromise",
    "Config",
    "ElementAPI",
    "JSAPIVar",
    "LambdaVar",
    "LiteralLambdaVar",
    "ModelWrapper",
    "ModelWrapperSSRM",
    "PassthroughAPI",
    "PromiseVar",
    "ag_chart",
    "ag_grid",
    "arrow_func",
    "autocomplete",
    "collapse",
    "combobox",
    "components",
    "dates",
    "dnd",
    "flow",
    "is_static",
    "json_input",
    "loading_overlay",
    "mantine",
    "map",
    "model_wrapper",
    "model_wrapper_ssrm",
    "multi_select",
    "number_formatter",
    "pill",
    "pills_input",
    "ring_progress",
    "semi_circle_progress",
    "spoiler",
    "static",
    "tags_input",
    "timeline",
    "tree",
]
